@@ -1,6 +1,6 @@
-#package Sim::OPT::Interlinear;
+package Sim::OPT::Interlinear;
 
-# INTERLINEAR, v. 0.009
+# INTERLINEAR, v. 0.0011
 # Author: Gian Luca Brunetti, Politecnico di Milano. (gianluca.brunetti@polimi.it)
 # Copyright reserved.  2018.
 # GPL License 3.0 or newer.
@@ -20,13 +20,13 @@ use feature 'say';
 no strict;
 no warnings;
 
-#use Sim::OPT;
-#use Sim::OPT::Morph;
-#use Sim::OPT::Sim;
-#use Sim::OPT::Report;
-#use Sim::OPT::Descend;
-#use Sim::OPT::Takechance;
-#use Sim::OPT::Parcoord3d;
+use Sim::OPT;
+use Sim::OPT::Morph;
+use Sim::OPT::Sim;
+use Sim::OPT::Report;
+use Sim::OPT::Descend;
+use Sim::OPT::Takechance;
+use Sim::OPT::Parcoord3d;
 
 our @ISA = qw( Exporter );
 our @EXPORT = qw( interlinear, interstart );
@@ -41,7 +41,7 @@ $ABSTRACT = 'Interlinear is a program for building metamodels from incomplete, m
 
 my $maxloops= 1000;
 my $sourcefile = "/home/luca/int/starcloud.csv";
-my $newfile = $sourcefile . "_new.csv";
+my $newfile = $sourcefile . "_cy_new.csv";
 my $report = $newfile . "_report.txt";
 my @mode = ( "wei" ); # #"wei" is weighted gradient linear interpolation of the nearest neighbours.
 #my @mode = ( "near" ); # "nea" means "nearest neighbour"
@@ -72,7 +72,7 @@ my $minreq_forcalc = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIAB
 my $minreq_formerge = 0; # THIS VALUE SPECIFIES A STRENGTH VALUE (LEVEL OF RELIABILITY) TELLING HOW WELL-ROOTED IN SIMULATED REALITY A DERIVED POINT MUST BE FOR MERGING IT IN THE CALCULATIONS FOR MERGING IT IN THE METAMODEL. A VALUE BETWEEN 1 (JUST SIMULATED POINT) AND 0 (SIMULATED POINTS AND "META"POINTS WITH THE SAME RIGHT ) MUST BE SPECIFIED. If 0, no entry barrier.
 my $minimumcertain = 0; # WHAT IS THE MINIMUM LEVEL OF STRENGTH (LEVEL OF RELIABILITY) REQUIRED TO USE A DATUM TO BUILD UPON IT. IT DEPENDS ON THE DISTANCE FROM THE ORIGINS OF THE DATUM. THE LONGER THE DISTANCE, THE SMALLER THE STRENGTH (WHICH IS INDEED INVERSELY PROPORTIONAL). A STENGTH VALUE OF 1 IS OF A SIMULATED DATUM, NOT OF A DERIVED DATUM. If 0, no entry barrier.
 my $minimumhold = 1; # WHAT IS THE MINIMUM LEVEL OF STRENGTH (LEVEL OF RELIABILITY) REQUIRED FOR NOT AVERAGING A DATUM WITH ANOTHER, DERIVED DATUM. USUALLY IT HAS TO BE KEPT EQUAL TO $minimimcertain.  If 1, ONLY THE MODEL DATA ARE NOT SUBSTITUTABLE IN THE METAMODEL.
-my $condweight = "no"; # THIS CONDITIONS TELLS IF THE STRENGTH (LEVEL OF RELIABILITY) OF THE GRADIENTS HAS TO BE CUMULATIVELY TAKEN INTO ACCOUNT IN THE WEIGHTING CALCULATIONS.
+my $condweight = "yes"; # THIS CONDITIONS TELLS IF THE STRENGTH (LEVEL OF RELIABILITY) OF THE GRADIENTS HAS TO BE CUMULATIVELY TAKEN INTO ACCOUNT IN THE WEIGHTING CALCULATIONS.
 
 #######################################################################
 
@@ -266,7 +266,7 @@ sub pythagoras
 
 sub weightvals1
 {
-  my ( $elt1, $boxgrads_ref, $boxdists_ref, $boxors_ref, $minreq_forcalc, $factbag_ref, $levbag_ref, $stepbag_ref, $elt2, $factlevels_ref, $maxdist, $el3, $elt3, $condweight ) = @_;
+  my ( $elt1, $boxgrads_ref, $boxdists_ref, $boxors_ref, $minreq_forcalc, $factbag_ref, $levbag_ref, $stepbag_ref, $elt2, $factlevels_ref, $maxdist, $el3, $elt3, $condweight, $distn_ref, $el0, $elt0 ) = @_;
   my @boxgrads = @{ $boxgrads_ref }; #say $tee "weightvals1 \@boxgrads: " . dump( @boxgrads );
   my @boxdists = @{ $boxdists_ref }; #say $tee "weightvals1 \@boxdists: " . dump( @boxdists );
   my @boxors = @{ $boxors_ref }; #say $tee "weightvals1 \@boxors: " . dump( boxors );
@@ -274,6 +274,8 @@ sub weightvals1
   my @levbag = @{ $levbag_ref }; #say $tee "weightvals12 \@levbag: " . dump( @levbag );
   my @stepbag = @{ $stepbag_ref }; #say $tee "weightvals12 \@stepbag: " . dump( @stepbag );
   my %factlevels = %{ $factlevels_ref }; #say $tee "weightvals12 \%factlevels: " . dump( %factlevels );
+  say $tee "\$maxdist: " .dump( $maxdist );
+  #say $tee "\$distn_ref: " .dump( $distn_ref );
   #say $tee "weightvals1 \$maxdist: " . dump( $maxdist );
 
   #if ( ref( $minreq_forcalc ) )
@@ -293,7 +295,9 @@ sub weightvals1
   my @newboxdists;
   foreach my $or ( @boxors )
   {
-    my $hsh_ref = calcdist( $elt1, $or, \%factlevels, $maxdist, $el3, $elt3, $condweight ); #say $tee "555 \$hsh_ref: " . dump( $hsh_ref );
+    say $tee "weightvals12 \@boxors: " . dump( @boxors );
+    my $hsh_ref = calcdist( { elt1 => $elt1, el1 => $or, factlevels_ref => \%factlevels, maxdist => $maxdist,
+      el3 => $el3, elt3 => $elt3, condweight => $condweight, distn_ref => $distn_ref, el0 => $el0, elt0 => $elt0 } ); #say $tee "555 \$hsh_ref: " . dump( $hsh_ref );
     my %hsh = %{ $hsh_ref }; #say $tee "555 \$elt1: " . dump( $elt1 ); #say $tee "555 \$or: " . dump( $or );
     my $dist = $hsh{dist}; #say $tee "555 \$dist: " . dump( $dist );
     push ( @newboxdists, $dist );
@@ -351,6 +355,7 @@ sub weightvals1
 } #END SUB weightvals1
 
 
+
 sub weightvals_merge
 {
   my ( $vals_ref, $strengths_ref, $minreq_formerge, $maxdist ) = @_;
@@ -421,138 +426,168 @@ sub weightvals_merge
 
 sub calcdist
 {
-  my ( $el1, $elt1, $factlevels_ref, $maxdist, $el3, $elt3, $condweight ) = @_;
+  my ( $h_ref ) = @_;
+  my %h = %{ $h_ref };
+
+  my $el1 = $h{el1}; #say $tee "OUT CALCDIST \$el1: " .dump( \$el1 );
+  my $elt1 = $h{elt1}; #say $tee "OUT CALCDIST \$elt1: " .dump( \$elt1 );
+  my $factlevels_ref = $h{factlevels_ref}; #say $tee "OUT CALCDIST \$factlevels_ref: " .dump( \$factlevels_ref );
+  my $maxdist = $h{maxdist}; #say $tee "OUT CALCDIST \$maxdist: " .dump( \$maxdist );
+  my $el3 = $h{el3}; #say $tee "OUT CALCDIST \$el3: " .dump( \$el3 );
+  my $elt3 = $h{elt3}; #say $tee "OUT CALCDIST \$elt3: " .dump( \$elt3 );
+  my $condweight = $h{condweight}; #say $tee "OUT CALCDIST \$condweight: " .dump( \$condweight );
+  my $distn_ref = $h{distn_ref}; #say $tee "OUT CALCDIST \$distn_ref: " .dump( \$distn_ref );
+  my $el0 = $h{el0}; #say $tee "OUT CALCDIST \$el0: " .dump( \$el0 );
+  my $elt0 = $h{elt0}; #say $tee "OUT CALCDIST \$elt0: " .dump( \$elt0 );
+  my $signal = $h{signal}; #say $tee "OUT CALCDIST \$signal: " .dump( \$signal );
+
   my %factlevels = %{ $factlevels_ref };
-  my @diff1 = diff( \@{ $el1 }, \@{ $elt1 } );
-  my $dist;
-  my ( @factbag, @levbag, @stepbag, @da1, @da1par, @da2, @da2par, @diff2 );
+  my %distn = %{ $distn_ref }; #say $tee "OUT0 CALCDIST \%distn: " .dump( \%distn );
+  if ( ( $distn{$el0}{$elt0} eq "y" ) or ( $signal eq "signal" ) )
+  { #say $tee "IN002 CALCDIST \%distn: " .dump( \%distn );
+    my @diff1 = diff( \@{ $el1 }, \@{ $elt1 } );
+    my $dist;
+    my ( @factbag, @levbag, @stepbag, @da1, @da1par, @da2, @da2par, @diff2 );
 
-  #say $tee "777\scalar( \@diff1 ): " . dump( scalar( @diff1 ) );
-  if ( scalar( @diff1 ) > 0 )
-  {
-    @diff2 = diff( \@{ $elt1 } , \@{ $el1 } ); #say $tee "777b \@diff2: " . dump( @diff2 );
+    #say $tee "777\scalar( \@diff1 ): " . dump( scalar( @diff1 ) );
+    if ( scalar( @diff1 ) > 0 )
+    {
+      @diff2 = diff( \@{ $elt1 } , \@{ $el1 } ); #say $tee "777b \@diff2: " . dump( @diff2 );
 
-    my %h1 = map { split( /-/ , $_ ) } @diff1; #say $tee "THEN \%h1: " . dump( %h1 );
-    @da1 = keys %h1; #say $tee "THEN \@da1: " . dump( @da1 );
-    @da1par = values %h1; #say $tee "\@da1par: " . dump( @da1par );
-    my %h2 = map { split( /-/ , $_ ) } @diff2; #say $tee "THEN \%h2: " . dump( %h2 );
-    @da2 = keys %h2; #say $tee "THEN \@da2: " . dump( @da2 );
-    @da2par = values %h2; #say $tee "\@da2par: " . dump( @da2par );
+      my %h1 = map { split( /-/ , $_ ) } @diff1; #say $tee "THEN \%h1: " . dump( %h1 );
+      @da1 = keys %h1; #say $tee "THEN \@da1: " . dump( @da1 );
+      @da1par = values %h1; #say $tee "\@da1par: " . dump( @da1par );
+      my %h2 = map { split( /-/ , $_ ) } @diff2; #say $tee "THEN \%h2: " . dump( %h2 );
+      @da2 = keys %h2; #say $tee "THEN \@da2: " . dump( @da2 );
+      @da2par = values %h2; #say $tee "\@da2par: " . dump( @da2par );
 
-    my $c = 0;
-    foreach my $e ( @da1par )
-    { #say $tee "\$e: " . dump( $e );
-      my $e = $da1par[$c];
-      my $ei = $da2par[$c]; #say $tee "22 \$ei: " . dump( $ei );
-      my $fact = $da1[$c]; #say $tee "22 \$fact: " . dump( $fact );
-      my $diffpar = abs( $e - $ei ); #say $tee "22 \$diffpar: " . dump( $diffpar ); say $tee "22 \$factlevels{stepsizes}{\$fact}: " . dump( $factlevels{stepsizes}{$fact} );
-      if ( ( $fact ne "" ) and ( $diffpar ne "" ) and ( $factlevels{stepsizes}{$fact} ne "" ) )
-      {
-        push ( @factbag, $fact );
-        push ( @levbag, $diffpar );
-        push ( @stepbag, $factlevels{stepsizes}{$fact} );
+      my $c = 0;
+      foreach my $e ( @da1par )
+      { #say $tee "\$e: " . dump( $e );
+        my $e = $da1par[$c];
+        my $ei = $da2par[$c]; #say $tee "22 \$ei: " . dump( $ei );
+        my $fact = $da1[$c]; #say $tee "22 \$fact: " . dump( $fact );
+        my $diffpar = abs( $e - $ei ); #say $tee "22 \$diffpar: " . dump( $diffpar ); say $tee "22 \$factlevels{stepsizes}{\$fact}: " . dump( $factlevels{stepsizes}{$fact} );
+        if ( ( $fact ne "" ) and ( $diffpar ne "" ) and ( $factlevels{stepsizes}{$fact} ne "" ) )
+        {
+          push ( @factbag, $fact );
+          push ( @levbag, $diffpar );
+          push ( @stepbag, $factlevels{stepsizes}{$fact} );
+        }
+        #say $tee "777b \@factbag : " . dump( @factbag ); say $tee "777b \@levbag : " . dump( @levbag ); say $tee "777b \@stepbag : " . dump( @stepbag );
+        $c++;
       }
-      #say $tee "777b \@factbag : " . dump( @factbag ); say $tee "777b \@levbag : " . dump( @levbag ); say $tee "777b \@stepbag : " . dump( @stepbag );
-      $c++;
     }
-  }
-  unless ( ( scalar( @levbag ) == 0 ) or ( scalar( @factbag ) == 0 ) or ( scalar( @stepbag ) == 0 ) )
-  {
-    my $ordist = scalar( @levbag ); #say $tee "\$ordist: " . dump( $ordist );
-    my $rawdist = pythagoras( \@factbag, \@levbag, \@stepbag ); #say $tee "776b\$rawdist: " . dump( $rawdist );
-    my ( $dist, $strength );
-
-    if ( $maxdist ne "" )
+    unless ( ( scalar( @levbag ) == 0 ) or ( scalar( @factbag ) == 0 ) or ( scalar( @stepbag ) == 0 ) )
     {
-      $dist = ( $rawdist / $maxdist );
-      $strength = ( 1 - $dist );
-    }
+      my $ordist = scalar( @levbag ); #say $tee "\$ordist: " . dump( $ordist );
+      my $rawdist = pythagoras( \@factbag, \@levbag, \@stepbag ); #say $tee "776b\$rawdist: " . dump( $rawdist );
+      my ( $dist, $strength );
 
-    if ( ( $el3 ne "" ) and ( $elt3 ne "" ) and ( $condweight = "yes") )
-    {
-      $strength = ( $strength * $el3 * $elt3 );
+      if ( $maxdist ne "" )
+      {
+        $dist = ( $rawdist / $maxdist );
+        $strength = ( 1 - $dist );
+      }
+
+      if ( ( $el3 ne "" ) and ( $elt3 ne "" ) and ( $condweight = "yes") )
+      {
+        $strength = ( $strength * $el3 * $elt3 );
+      }
+      my $h_r = { dist => $dist, strength => $strength, ordist => $ordist, rawdist => $rawdist, diff1 => \@diff1,
+        diff2 => \@diff2, da1 => \@da1, da2 => \@da2, da1par => \@da1par, da2par => \@da2par,
+        signal => $h{signal} }; #say $tee "779\$h_r: " . dump( $h_r );
+      return( $h_r );
     }
-    return( { dist => $disfactlevelst, strength => $strength, ordist => $ordist, rawdist => $rawdist, diff1 => \@diff1, diff2 => \@diff2, da1 => \@da1, da2 => \@da2, da1par => \@da1par, da2par => \@da2par } );
   }
 }
 
 
 sub calcdistgrad
 {
-  my ( $el1, $elt1, $factlevels_ref, $minreq, $maxdist, $el3, $elt3, $condweight ) = @_; #say $tee "777c\$el1: " . dump( $el1 ); say $tee "777c\$elt1: " . dump( $elt1 ); say $tee "777c\$minreq: " . dump( $minreq );
+  my ( $el1, $elt1, $factlevels_ref, $minreq, $maxdist, $el3, $elt3, $condweight, $distn_ref, $el0, $elt0 ) = @_; #say $tee "777c\$el1: " . dump( $el1 ); say $tee "777c\$elt1: " . dump( $elt1 ); say $tee "777c\$minreq: " . dump( $minreq );
   my %factlevels = %{ $factlevels_ref };
-  my @diff1 = diff( \@{ $el1 }, \@{ $elt1 } ); #say $tee "777\@diff1: " . dump( @diff1 );
-  my $dist;
-  my ( @factbag, @levbag, @stepbag, @da1, @da1par, @da2, @da2par, @diff2 );
-  #say $tee "777\scalar( \@diff1 ): " . dump( scalar( @diff1 ) ); say $tee "777\$minreq: " . dump( $minreq );
-  if ( ( scalar( @diff1 ) > 0 ) and ( scalar( @diff1 ) <= $minreq->[0] ) )
-  {
-    @diff2 = diff( \@{ $elt1 } , \@{ $el1 } ); #say $tee "777b \@diff2: " . dump( @diff2 ); say $tee "777\@diff1: " . dump( @diff1 );
+  my %distn = %{ $distn_ref }; #say $tee "OUT CALCDISTRGRAD\%distn: " .dump( \%distn );
+  if ( $distn{$el0}{$elt0} eq "y" )
+  { #say $tee "IN CALCDISTRGRAD \%distn: " .dump( \%distn );
+    my @diff1 = diff( \@{ $el1 }, \@{ $elt1 } ); #say $tee "777\@diff1: " . dump( @diff1 );
+    my $dist;
+    my ( @factbag, @levbag, @stepbag, @da1, @da1par, @da2, @da2par, @diff2 );
+    #say $tee "777\scalar( \@diff1 ): " . dump( scalar( @diff1 ) ); say $tee "777\$minreq: " . dump( $minreq );
+    if ( ( scalar( @diff1 ) > 0 ) and ( scalar( @diff1 ) <= $minreq->[0] ) )
+    {
+      @diff2 = diff( \@{ $elt1 } , \@{ $el1 } ); #say $tee "777b \@diff2: " . dump( @diff2 ); say $tee "777\@diff1: " . dump( @diff1 );
 
-    my %h1 = map { split( /-/ , $_ ) } @diff1; #say $tee "THEN \%h1: " . dump( %h1 );
-    @da1 = keys %h1; #say $tee "THEN \@da1: " . dump( @da1 );
-    @da1par = values %h1; #say $tee "\@da1par: " . dump( @da1par );
-    my %h2 = map { split( /-/ , $_ ) } @diff2; #say $tee "THEN \%h2: " . dump( %h2 );
-    @da2 = keys %h2; #say $tee "THEN \@da2: " . dump( @da2 );
-    @da2par = values %h2; #say $tee "\@da2par: " . dump( @da2par );
+      my %h1 = map { split( /-/ , $_ ) } @diff1; #say $tee "THEN \%h1: " . dump( %h1 );
+      @da1 = keys %h1; #say $tee "THEN \@da1: " . dump( @da1 );
+      @da1par = values %h1; #say $tee "\@da1par: " . dump( @da1par );
+      my %h2 = map { split( /-/ , $_ ) } @diff2; #say $tee "THEN \%h2: " . dump( %h2 );
+      @da2 = keys %h2; #say $tee "THEN \@da2: " . dump( @da2 );
+      @da2par = values %h2; #say $tee "\@da2par: " . dump( @da2par );
 
-    my $c = 0;
-    foreach my $lev1 ( @da1par )
-    { #say $tee "\$e: " . dump( $e );
-      my $da1 = $da1[$c];
-      my $i = 0;
-      foreach my $lev2 ( @da2par )
-      {
-        my $da2 = $da2[$i];
-        if ( $da1 == $da2 )
+      my $c = 0;
+      foreach my $lev1 ( @da1par )
+      { #say $tee "\$e: " . dump( $e );
+        my $da1 = $da1[$c];
+        my $i = 0;
+        foreach my $lev2 ( @da2par )
         {
-          my $fact = $da1;
-          my $diffpar = abs( $lev1 - $lev2 ); #say $tee "22 \$diffpar: " . dump( $diffpar );
-          if ( ( $diffpar <= $minreq->[1] ) and ( $fact ne "" ) and ( $diffpar ne "" ) and ( $factlevels{stepsizes}{$fact} ne "" ) )
+          my $da2 = $da2[$i];
+          if ( $da1 == $da2 )
           {
-            push ( @factbag, $fact );
-            push ( @levbag, $diffpar );
-            push ( @stepbag, $factlevels{stepsizes}{$fact} );
+            my $fact = $da1;
+            my $diffpar = abs( $lev1 - $lev2 ); #say $tee "22 \$diffpar: " . dump( $diffpar );
+            if ( ( $diffpar <= $minreq->[1] ) and ( $fact ne "" ) and ( $diffpar ne "" ) and ( $factlevels{stepsizes}{$fact} ne "" ) )
+            {
+              push ( @factbag, $fact );
+              push ( @levbag, $diffpar );
+              push ( @stepbag, $factlevels{stepsizes}{$fact} );
+            }
           }
+          $i++;
         }
-        $i++;
+        $c++;
       }
-      $c++;
     }
-  }
 
-  unless ( ( scalar( @levbag ) == 0 ) or ( scalar( @factbag ) == 0 ) or ( scalar( @stepbag ) == 0 ) )
-  {
-    my $ordist = scalar( @levbag ); #say $tee "\$ordist: " . dump( $ordist );
-    my $rawdist = pythagoras( \@factbag, \@levbag, \@stepbag ); #say $tee "777b\$rawdist: " . dump( $rawdist );
-    my ( $dist, $strength );
-
-    if ( $maxdist ne "" )
+    unless ( ( scalar( @levbag ) == 0 ) or ( scalar( @factbag ) == 0 ) or ( scalar( @stepbag ) == 0 ) )
     {
-      $dist = ( $rawdist / $maxdist );
-      $strength = ( 1 - $dist );
-    }
+      my $ordist = scalar( @levbag ); #say $tee "\$ordist: " . dump( $ordist );
+      my $rawdist = pythagoras( \@factbag, \@levbag, \@stepbag ); #say $tee "777b\$rawdist: " . dump( $rawdist );
+      my ( $dist, $strength );
 
-    if ( ( $el3 ne "" ) and ( $elt3 ne "" ) and ( $condweight = "yes") )
-    {
-      $strength = ( $strength * $el3 * $elt3 );
+      if ( $maxdist ne "" )
+      {
+        $dist = ( $rawdist / $maxdist );
+        $strength = ( 1 - $dist );
+      }
+
+      if ( ( $el3 ne "" ) and ( $elt3 ne "" ) and ( $condweight = "yes") )
+      {
+        $strength = ( $strength * $el3 * $elt3 );
+      }
+      return( { dist => $dist, strength => $strength, ordist => $ordist, rawdist => $rawdist, diff1 => \@diff1,
+        diff2 => \@diff2, da1 => \@da1, da2 => \@da2, da1par => \@da1par, da2par => \@da2par } );
     }
-    return( { dist => $dist, strength => $strength, ordist => $ordist, rawdist => $rawdist, diff1 => \@diff1, diff2 => \@diff2, da1 => \@da1, da2 => \@da2, da1par => \@da1par, da2par => \@da2par } );
   }
 }
 
 
 sub calcmaxdist
 {
-  my ( $arr_ref, $factlevels_ref ) = @_;
-  my @arr = @{ $arr_ref };
+  my ( $arr_ref, $factlevels_ref, $distn_ref ) = @_;
+  my @arr = @{ $arr_ref }; #say $tee "00\@arr: " . dump( @arr ); say $tee "00\$arr_ref: " . dump( $arr_ref );
+  my %distn = %{ $distn_ref };
   my @rawdists;
   foreach my $el ( @arr )
   {
     foreach my $elt ( @arr )
-    {
-      my $hash_ref = calcdist( $el->[1], $elt->[1], $factlevels_ref );
-      my %hash = %{ $hash_ref }; #say $tee "00\%hash: " . dump( %hash );
+    { #say $tee "001el1 => $el->[1]: " . dump(  $el->[1] ); say $tee "001\elt1 => $elt->[1]: " . dump(  $elt->[1] );
+      #say $tee "001el3 => $el->[3]: " . dump(  $el->[3] ); say $tee "001\elt3 => $elt->[3]: " . dump(  $elt->[3] );
+      #say $tee "001el0 => $el->[0]: " . dump(  $el->[0] ); say $tee "001\elt0 => $elt->[0]: " . dump(  $elt->[0] );
+      my $h_r = calcdist( { el1 => $el->[1], elt1 => $elt->[1],
+        factlevels_ref => $factlevels_ref, distn_ref => $distn_ref, signal => "signal" } ); #say $tee "007\$h_r: " . dump( $h_r );
+      my %hash = %{ $h_r }; #say $tee "007\%hash: " . dump( %hash );
       push ( @rawdists, $hash{rawdist} );
     }
   }  #say $tee "00\@rawdists: " . dump( @rawdists );
@@ -561,14 +596,40 @@ sub calcmaxdist
 }
 
 
+sub exploredist
+{
+  my ( $arr_ref, $minreq_forgrad ) = @_;
+  my @arr = @{ $arr_ref };
+  my %distn;
+  foreach my $el ( @arr )
+  {
+    my $key =  $el->[0] ;
+    foreach my $elt ( @arr )
+    {
+      unless ( $el->[0] eq $elt->[0] )
+      {
+        my @diff1 = diff( \@{ $el->[1] }, \@{ $elt->[1] } );
+        my $scalar = scalar( @diff1 );
+        if ( ( $scalar > 0 ) and ( $scalar <= $minreq_forgrad->[0] ) )
+        {
+          $distn{$el->[0]}{$elt->[0]} = "y";
+        }
+      }
+    }
+  } #say $tee "IN EXPLORE \%distn: " .dump( \%distn );
+  return( \%distn );
+}
+
+
 sub wei
 {
   my ( $arr_ref, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count,
-    $factlevels_ref, $minreq_forgrad, $minreq_forinclusion, $minreq_forcalc, $minreq_formerge, $maxdist ) = @_;
+    $factlevels_ref, $minreq_forgrad, $minreq_forinclusion, $minreq_forcalc, $minreq_formerge, $maxdist, $distn_ref ) = @_;
   my @arr = @{ $arr_ref };
   #say $tee "ARR VERY_BEFORE: " . dump(@arr);
   my %factlevels = %{ $factlevels_ref };  #say $tee "AND \%factlevels: " . dump( %factlevels );
-  # my ( %magic, %wand, %spell, %bank );
+  my %distn = %{ $distn_ref }; #say $tee "\%distn: " .dump( \%distn );
+
 
   sub fillbank
   { #say $tee "NOW IN FILLBANK.";
@@ -584,7 +645,7 @@ sub wei
           { #say $tee "NOW CHECKING .";
 
             my $d_ref; #say $tee "\$el->[1] : " . dump( $el->[1] ); say $tee "\$elt->[1]: " . dump( $elt->[1] );
-            my ( $res_ref ) = calcdistgrad( $el->[1], $elt->[1], \%factlevels, $minreq_forgrad, $maxdist, $el->[3], $elt->[3], $condweight );
+            my ( $res_ref ) = calcdistgrad( $el->[1], $elt->[1], \%factlevels, $minreq_forgrad, $maxdist, $el->[3], $elt->[3], $condweight, \%distn, $el->[0], $elt->[0] );
 
             #if ( $res_ref ne "" )
             unless ( !keys %{ $res_ref } )
@@ -682,11 +743,11 @@ sub wei
                         {
                           if ( $orderedtrio eq $trio )
                           {
-                            push ( @{ $bank{$trio}{grads} }, (- $grad ) );
+                            push ( @{ $bank{$trio}{grads} }, ( - $grad ) );
                           }
                           else
                           {
-                            push ( @{ $bank{$trio}{grads} }, $grad );
+                            push ( @{ $bank{$trio}{grads} }, ( $grad ) );
                           }
                         }
                       }
@@ -706,11 +767,11 @@ sub wei
 
 
   my $bank_ref = fillbank;
-  my %bank =  %{ $bank_ref }; say $tee "\%bank: " . dump( %bank );
+  my %bank =  %{ $bank_ref }; #say $tee "\%bank: " . dump( %bank );
 
 
   sub cyclearr
-  { say $tee "NTH ARR: " . dump( @arr );
+  { #say $tee "NTH ARR: " . dump( @arr );
     my %wand;
     my $coun = 0;
     foreach my $el ( @arr )
@@ -792,15 +853,15 @@ sub wei
                       foreach my $e ( @da1par )
                       {
                         if ( $e ne "" )
-                        { say $tee "NOW IN5"; say $tee "33\$e: " . dump( $e );
-                          my $fact = $da1[$c]; say $tee "33\$fact: " . dump( $fact );
+                        { #say $tee "NOW IN5"; say $tee "33\$e: " . dump( $e );
+                          my $fact = $da1[$c]; #say $tee "33\$fact: " . dump( $fact );
 
                           my $i = 0;
                           foreach my $ei ( @da2par )
                           {
                             if ( $da1[$c] == $da2[$i] )
                             {
-                              my $diffpar = abs( $e - $ei ); say $tee "33\$diffpar: " . dump( $diffpar );
+                              my $diffpar = abs( $e - $ei ); #say $tee "33\$diffpar: " . dump( $diffpar );
                               #if ( ( $diffpar >= $minreq_forgrad->[1] ) and ( $diffpar > 0 ) )
                               if ( ( $diffpar <= $minreq_forgrad->[1] ) and ( $diffpar > 0 ) ) ############################à
                               {
@@ -813,15 +874,13 @@ sub wei
                           }
                         }
                         $c++;
-                      }
-                      say $tee "\@factbag: " . dump( @factbag ); say $tee "\@levbag: " . dump( @levbag ); say $tee "\@stepbag: " . dump( @stepbag );
+                      } #say $tee "\@factbag: " . dump( @factbag ); say $tee "\@levbag: " . dump( @levbag ); say $tee "\@stepbag: " . dump( @stepbag );
 
                       my ( $soughtinc, $totdist, $totstrength );
 
                       unless ( ( scalar( @boxgrads ) == 0 ) and ( scalar( @boxdists ) == 0 ) and ( scalar( @boxors ) == 0 ) )
                       {
-                        ( $soughtinc, $totdist, $totstrength ) = weightvals1( $elt->[1], \@boxgrads, \@boxdists, \@boxors, $minreq_forcalc, \@factbag, \@levbag, \@stepbag, $elt->[2], \%factlevels, $maxdist, $el->[3], $elt->[3], $condweight );
-
+                        ( $soughtinc, $totdist, $totstrength ) = weightvals1( $elt->[1], \@boxgrads, \@boxdists, \@boxors, $minreq_forcalc, \@factbag, \@levbag, \@stepbag, $elt->[2], \%factlevels, $maxdist, $el->[3], $elt->[3], $condweight, $distn_ref, $el->[0], $elt->[0] );
 
                         unless ( ( $soughtinc eq "" ) or ( $totdist eq "" ) )
                         {
@@ -845,10 +904,10 @@ sub wei
                         #  say $tee "NOT PUSHING INTO WAND ";
                         #}
                       }
-                      else
-                      {
-                        say $tee "NOT WEIGHTVALS1ING ";
-                      }
+                      #else
+                      #{
+                      #  say $tee "NOT WEIGHTVALS1ING ";
+                      #}
                     }
                   }
                   $co++
@@ -859,14 +918,12 @@ sub wei
           }
         }
       }
-    }
-    say $tee "\%wand IN: " . dump( %wand );
+    } #say $tee "\%wand IN: " . dump( %wand );
     return( \%wand );
   }
 
   my $wand_ref = cyclearr;
-  my %wand = %{ $wand_ref };
-  say $tee "\%wand OUT: " . dump( %wand );
+  my %wand = %{ $wand_ref }; #say $tee "\%wand OUT: " . dump( %wand );
 
 
   my @limb0;
@@ -1337,16 +1394,23 @@ sub interlinear
   close SOURCEFILE;
 
   my $aarr_ref;
+  say $tee "PREPARING THE DATASERIES. ";
   ( $aarr_ref, $optformat ) = preparearr( \@lines );
 
   my @aarr = @{ $aarr_ref };
 
+  say $tee "CHECKING LEVELS AND FACTORS. ";
   my %factlevels = prepfactlev( @aarr );
 
   my ( $factlev_ref ) = tellstepsize( \%factlevels );
   my %factlev = %{ $factlev_ref }; #say $tee "\%factlev: " . dump( %factlev );
 
-  my $maxdist = calcmaxdist( \@aarr, \%factlev ); #say $tee "001\$maxdist: " . dump( $maxdist );
+  say $tee "PRELIMINARLY EXPLORING THE SPACE. ";
+
+  my $distn_ref = exploredist( \@aarr, $minreq_forgrad );
+  my %distn = %{ $distn_ref };
+
+  my $maxdist = calcmaxdist( \@aarr, \%factlev, \%distn ); #say $tee "001\$maxdist: " . dump( $maxdist );
 
   my $count = 0;
   while ( $count < $maxloops )
@@ -1367,17 +1431,9 @@ sub interlinear
     my ( @limbo_wei, @limbo_purelin, @limbo_near, @limbo, %bank, %wand );
 
 
-    #if ( ( $mode__ eq "vault" ) or ( $mode__ eq "mix" ) )
-    #{
-    #  @limbo_vault = vault( \@arr, $relaxlimit, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, \%factlev );
-    #}
-    #say $tee "OBTAINED limbo_vault: " . dump( @limbo_vault );
-    #say $tee "THERE ARE " . scalar( @limbo_vault ) . " ITEMS IN THIS LOOP , NUMBER " . ( $count + 1 ). ", 1, FOR GLOBALLY WEIGHTED GRADIENT INTERPOLATION.";
-
-
     if ( ( $mode__ eq "wei" ) or ( $mode__ eq "mix" ) )
     {
-      @limbo_wei = wei( \@arr, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, \%factlev, $minreq_forgrad, $minreq_forinclusion, $minreq_forcalc, $minreq_formerge, $maxdist );
+      @limbo_wei = wei( \@arr, $relaxmethod, $overweightnearest, $parconcurrencies, $instconcurrencies, $count, \%factlev, $minreq_forgrad, $minreq_forinclusion, $minreq_forcalc, $minreq_formerge, $maxdist, $distn_ref );
       say $tee "THERE ARE " . scalar( @limbo_wei ) . " ITEMS IN THIS LOOP , NUMBER " . ( $count + 1 ). ", 1, FOR WEIGHTED GRADIENT INTERPOLATION OF THE NEAREST NEIGHBOUR.";
     }
     #say $tee "OBTAINED LIMBO_WEI: " . dump( @limbo_wei );
@@ -1419,17 +1475,17 @@ sub interlinear
     #  @limbo = mixlimbo( \@limbo_prov, \@limbo_near, $presence, $linearprecedence, \@weights );
     #}
 
-    say $tee "OBTAINED LIMBO: " . dump( @limbo );
+    #say $tee "OBTAINED LIMBO: " . dump( @limbo );
     say $tee "MIXING THE ARRAY UPDATES " . ( $count + 1 ) . " for $sourcefile";
     say $tee "THERE ARE " . scalar( @limbo ) . " ITEMS COMING OUT FROM THIS MIX " . ( $count + 1 );
 
     if ( ( scalar( @limbo ) == 0 ) )
     {
-      say $tee "ARR END: " . dump( @arr );
+      #say $tee "ARR END: " . dump( @arr );
       last;
     }
 
-    say $tee "ARR BEFORE: " . dump( @arr );
+    #say $tee "ARR BEFORE: " . dump( @arr );
     foreach my $el ( @limbo )
     { #say $tee "EL AGAIN: " . dump( $el );
       foreach $elt ( @arr )
@@ -1464,7 +1520,7 @@ sub interlinear
     }
     @arr2 = @arr ;
     say $tee "INSERTING THE ARRAY UPDATES " . ( $count + 1 ) . " for $sourcefile";
-    say $tee "ARR AFTER: " . dump( @arr );
+    #say $tee "ARR AFTER: " . dump( @arr );
     $count++;
   }
   return( @arr );
